;**************************************************************************
;*                                                                        *
;* Program     : 240064-A2                                                *
;* Title       : 240 column x 64 row LCD Graphic Module Driving Program   *
;*                                                                        *
;* MCU         : AT89C52, 12MHz                                           *
;* Programmer  : Stephen Yu, Oct. 2002                                    *
;* File name   : 240064a2.asm                                             *
;* Eval. Board : LKEB-G03-A                                               *
;* Description : Drive 240x64 LCD graphic module with LCD controller      *
;*               T6963C                                                   *
;* LCD Module  : LKG-240064-A2 or equivalent                              *
;* Assembler   : AD2500 or equivalent                                     *
;* Copyright   : Lcdmod Kit                                               *
;* URL         : http://www.lcdmodkit.com                                 *
;**************************************************************************

;***************************************
;*      MCU <-> LCM pin connection     *
;*                                     *
;* P1.0 - 1.7 <-> DB0 -DB7             *
;***************************************

FS      REG     P3.0
RST     REG     P3.1
CLICK   REG     P3.2
C/D     REG     P3.3
CE      REG     P3.4
RD      REG     P3.5
WR      REG     P3.6

COM     EQU     32H
DAT1    EQU     30H
DAT2    EQU     31H
MARK    EQU     33H
O_Y     EQU     35H
O_X     EQU     36H
COUNT   EQU     34H
TEMP1   EQU     37H
TEMP2   EQU     38H
TEMP3   EQU     39H
TEMP4   EQU     3AH

        ORG     00H
START:
        SETB    RST          ;
        CLR     FS           ; SET FONT TO 8x8
        CLR     CE           ; CHIP ENABLE
        LCALL   INIT         ; CONTROLLER INITIALIZATION
MAIN:
        LCALL   ALL_ON       ; ALL PIXEL ON
        CALL    BUTTON

        LCALL   ALL_OFF      ; ALL PIXEL OFF
        CALL    BUTTON

        MOV     COM,#98H     ; ALTERNATE COLUMN ON
        LCALL   C_DW2
        MOV     O_X,#00H
        MOV     O_Y,#00H
        LCALL   COLUMN
        LCALL   BUTTON

        MOV     COM,#98H     ; ALTERNATE ROW ON
        LCALL   C_DW2
        MOV     O_X,#00H     ; X COOR = 0
        MOV     O_Y,#00H     ; Y COOR = 0
        LCALL   ROW
        LCALL   BUTTON

        MOV     COM,#98H     ; 1x1 CHECKER BOARD
        LCALL   C_DW2
        MOV     O_X,#00H     ; X COOR = 0
        MOV     O_Y,#00H     ; Y COOR = 0
        LCALL   S_CHECKER
        LCALL   BUTTON

        MOV     COM,#98H     ; 2x2 CHECKER BOARD
        LCALL   C_DW2
        MOV     O_X,#00H     ; X COOR = 0
        MOV     O_Y,#00H     ; Y COOR = 0
        LCALL   M_CHECKER
        LCALL   BUTTON

        MOV     COM,#98H     ; 4x4 CHECKER BOARD
        LCALL   C_DW2
        MOV     O_X,#00H     ; X COOR = 0
        MOV     O_Y,#00H     ; Y COOR = 0
        LCALL   B_CHECKER
        LCALL   BUTTON

        MOV     COM,#98H     ; DISPLAY AN IMAGE
        LCALL   C_DW2
        MOV     O_X,#00H     ; X COOR = 0
        MOV     O_Y,#00H     ; Y COOR = 0
        LCALL   GRAPH
        LCALL   BUTTON

        JMP     MAIN

INIT:                        ; ( PROCEDRUE FOR INITIALIZATION )
        MOV     DAT1,#00H
        MOV     DAT2,#00H
        MOV     COM,#40H     ; SET TEXT HOME ADDRESS TO 0000H
        LCALL   C_DW
        MOV     DAT1,#20H
        MOV     DAT2,#00H
        MOV     COM,#41H     ; SET TEXT AREA TO 32 COLUMNS
        LCALL   C_DW
        MOV     DAT1,#00H
        MOV     DAT2,#08H
        MOV     COM,#42H     ; SET GRAPHIC HOME ADDRESS TO 0800H
        LCALL   C_DW
        MOV     DAT1,#20H
        MOV     DAT2,#00H
        MOV     COM,#43H     ; SET GRAPHIC AREA TO 32 COLUMNS
        LCALL   C_DW
        MOV     COM,#0A7H    ; SET CURSOR TO 8-LINE CURSOR
        LCALL   C_DW2
        MOV     COM,#80H     ; DISPLAY MODE AS "OR" MODE
        LCALL   C_DW2
        MOV     COM,#9CH     ; TEXT ON, GRAPHIC ON, CURSOR DISPLAY OFF, CURSOR BLINK OFF
        LCALL   C_DW2
        RET

ALL_OFF:                     ; ( PROCEDRUE FOR PIXEL OFF )
        MOV     DAT1,#00H
        MOV     DAT2,#00H
        MOV     COM,#24H     ; SET ADDRESS POINTER TO 0000H
        LCALL   C_DW
        MOV     R3,#00H      ; SET Y COUNTER = 256
        MOV     R4,#20H      ; SET X COUNTER = 32
        MOV     COM,#0B0H    ; SET DATA AUTO WRITE
        LCALL   C_DW2
ALL_OFF1:
        LCALL   CS_3
        MOV     A,#00H       ; PUT 00000000 TO DATA BYTE
        LCALL   C_DW3
        DJNZ    R3,ALL_OFF1
        DJNZ    R4,ALL_OFF1
        MOV     COM,#0B2H    ; RESET AUTO
        LCALL   C_DW2
        RET

ALL_ON:                      ; ( PROCEDRUE FOR PIXEL ON )
        MOV     DAT1,#00H
        MOV     DAT2,#00H
        MOV     COM,#24H     ; SET ADDRESS POINTER TO 0000H
        LCALL   C_DW
        MOV     R3,#00H      ; SET Y COUNTER = 256
        MOV     R4,#20H      ; SET X COUNTER = 32
        MOV     COM,#0B0H    ; SET DATA AUTO WRITE
        LCALL   C_DW2
ALL_ON1:
        LCALL   CS_3
        MOV     A,#FFH       ; PUT 11111111 TO DATA BYTE
        LCALL   C_DW3
        DJNZ    R3,ALL_ON1
        DJNZ    R4,ALL_ON1
        MOV     COM,#0B2H    ; RESET AUTO
        LCALL   C_DW2
        RET

COLUMN:                      ; ( PROCEDRUE FOR ALTERNATE COLUMN ON )
        MOV     TEMP1,O_Y
        MOV     TEMP2,O_X
        MOV     R3,#64       ; SET Y COUNTER = 64
COLUMN1:
        MOV     A,O_Y        ; CALCULATE THE RAM ADDRESS
        MOV     B,#20H
        MUL     AB
        ADD     A,O_X
        MOV     O_X,A
        MOV     A,B
        ADDC    A,#08H
        MOV     O_Y,A
        MOV     R4,#32       ; SET X COUNTER = 32
COLUMN2:
        MOV     DAT1,O_X
        MOV     DAT2,O_Y
        MOV     COM,#24H     ; SET ADDRESS POINTER
        LCALL   C_DW

        MOV     DAT2,#AAH    ; PUT 10101010 TO DATA BYTE
        MOV     COM,#0C0H    ; DATA WRITE AND INCREMENT ADDRESS POINTER
        LCALL   C_DW1

        MOV     A,O_X        ; RE-CALCULATE THE RAM ADDRESS
        INC     A
        MOV     O_X,A
        DJNZ    R4,COLUMN2

        MOV     A,TEMP1
        INC     A
        MOV     TEMP1,A
        MOV     O_Y,TEMP1
        MOV     O_X,TEMP2
        DJNZ    R3,COLUMN1
        RET

ROW:                         ; ( PROCEDRUE FOR ALTERNATE ROW ON )
        MOV     TEMP1,O_Y
        MOV     TEMP2,O_X
        MOV     R3,#32       ; SET Y COUNTER = 32
ROW1:
        MOV     A,O_Y        ; CALCULATE THE RAM ADDRESS
        MOV     B,#20H
        MUL     AB
        ADD     A,O_X
        MOV     O_X,A
        MOV     A,B
        ADDC    A,#08H
        MOV     O_Y,A

        MOV     R4,#32       ; SET X COUNTER = 32
ROW2:
        MOV     DAT1,O_X
        MOV     DAT2,O_Y
        MOV     COM,#24H     ; SET ADDRESS POINTER
        LCALL   C_DW

        MOV     DAT2,#FFH    ; PUT 11111111 TO DATA BYTE
        MOV     COM,#0C0H    ; DATA WRITE AND INCREMENT ADDRESS POINTER
        LCALL   C_DW1

        MOV     A,O_X        ; RE-CALCULATE THE RAM ADDRESS
        INC     A
        MOV     O_X,A
        DJNZ    R4,ROW2

        MOV     A,TEMP1
        INC     A
        MOV     TEMP1,A
        MOV     O_Y,TEMP1

        MOV     O_X,TEMP2

        MOV     R4,#32       ; RE-SET X COUNTER = 32
ROW3:
        MOV     A,O_Y        ; RE-CALCULATE THE RAM ADDRESS
        MOV     B,#20H
        MUL     AB
        ADD     A,O_X
        MOV     O_X,A
        MOV     A,B
        ADDC    A,#08H
        MOV     O_Y,A

ROW4:
        MOV     DAT1,O_X
        MOV     DAT2,O_Y
        MOV     COM,#24H     ; SET ADDRESS POINTER
        LCALL   C_DW

        MOV     DAT2,#00H    ; PUT 00000000 TO DATA BYTE
        MOV     COM,#0C0H    ; DATA WRITE AND INCREMENT ADDRESS POINTER
        LCALL   C_DW1

        MOV     A,O_X        ; RE-CALCULATE THE RAM ADDRESS
        INC     A
        MOV     O_X,A
        DJNZ    R4,ROW4

        MOV     A,TEMP1
        INC     A
        MOV     TEMP1,A
        MOV     O_Y,TEMP1

        MOV     O_X,TEMP2
        DJNZ    R3,ROW1
        RET

S_CHECKER:                   ; ( PROCEDRUE FOR 1x1 CHEKER BOARD )
        MOV     TEMP1,O_Y
        MOV     TEMP2,O_X
        MOV     R3,#32       ; SET Y COUNTER = 32
S_CHECKER1:
        MOV     A,O_Y        ; CALCULATE THE RAM ADDRESS
        MOV     B,#20H
        MUL     AB
        ADD     A,O_X
        MOV     O_X,A
        MOV     A,B
        ADDC    A,#08H
        MOV     O_Y,A

        MOV     R4,#32       ; SET X COUNTER = 32
S_CHECKER2:
        MOV     DAT1,O_X
        MOV     DAT2,O_Y
        MOV     COM,#24H     ; SET ADDRESS POINTER
        LCALL   C_DW

        MOV     DAT2,#AAH    ; PUT 10101010 TO DATA BYTE
        MOV     COM,#0C0H    ; DATA WRITE AND INCREMENT ADDRESS POINTER
        LCALL   C_DW1

        MOV     A,O_X        ; RE-CALCULATE THE RAM ADDRESS
        INC     A
        MOV     O_X,A
        DJNZ    R4,S_CHECKER2

        MOV     A,TEMP1
        INC     A
        MOV     TEMP1,A
        MOV     O_Y,TEMP1

        MOV     O_X,TEMP2

        MOV     R4,#32       ; RE-SET X COUNTER = 32
S_CHECKER3:
        MOV     A,O_Y        ; RE-CALCULATE THE RAM ADDRESS
        MOV     B,#20H
        MUL     AB
        ADD     A,O_X
        MOV     O_X,A
        MOV     A,B
        ADDC    A,#08H
        MOV     O_Y,A

S_CHECKER4:
        MOV     DAT1,O_X
        MOV     DAT2,O_Y
        MOV     COM,#24H     ; SET ADDRESS POINTER
        LCALL   C_DW

        MOV     DAT2,#55H    ; PUT 01010101 TO DATA BYTE
        MOV     COM,#0C0H    ; DATA WRITE AND INCREMENT ADDRESS POINTER
        LCALL   C_DW1

        MOV     A,O_X        ; RE-CALCULATE THE RAM ADDRESS
        INC     A
        MOV     O_X,A
        DJNZ    R4,S_CHECKER4

        MOV     A,TEMP1
        INC     A
        MOV     TEMP1,A
        MOV     O_Y,TEMP1

        MOV     O_X,TEMP2
        DJNZ    R3,S_CHECKER1
        RET

M_CHECKER:                   ; ( PROCEDRUE FOR 2x2 CHEKER BOARD )
        MOV     DAT1,O_X     ; CALCULATE THE RAM ADDRESS
        MOV     A,O_Y
        ADD     A,#08H
        MOV     DAT2,A
        MOV     COM,#24H     ; SET ADDRESS POINTER
        LCALL   C_DW
        MOV     R3,#16       ; SET Y COUNTER = 16
        MOV     R4,#64       ; SET X COUTNER = 64
        MOV     COM,#0B0H    ; SET DATA AUTO WRITE
        LCALL   C_DW2
M_CHECKER1:
        LCALL   CS_3
        MOV     A,#CCH       ; PUT 11001100 TO DATA BYTE
        LCALL   C_DW3
        DJNZ    R4,M_CHECKER1
        MOV     R4,#64       ; RE-SET X COUNTER = 64
M_CHECKER2:
        LCALL   CS_3
        MOV     A,#33H       ; PUT 00110011 TO DATA BYTE
        LCALL   C_DW3
        DJNZ    R4,M_CHECKER2
        MOV     R4,#64       ; RE-SET X OOUTNER = 64
        DJNZ    R3,M_CHECKER1
        MOV     COM,#0B2H    ; RESET AUTO
        LCALL   C_DW2
        RET

B_CHECKER:                   ; ( PROCEDRUE FOR 4x4 CHEKER BOARD )
        MOV     DAT1,O_X
        MOV     A,O_Y
        ADD     A,#08H
        MOV     DAT2,A
        MOV     COM,#24H     ; SET ADDRESS POINTER
        LCALL   C_DW
        MOV     R3,#8        ; SET Y COUNTER = 8
        MOV     R4,#128      ; SET X XOUNTER = 128
        MOV     COM,#0B0H    ; SET DATA AUTO WRITE
        LCALL   C_DW2
B_CHECKER1:
        LCALL   CS_3
        MOV     A,#F0H       ; PUT 11110000 TO DATA BYTE
        LCALL   C_DW3
        DJNZ    R4,B_CHECKER1
        MOV     R4,#128      ; RE-SET X COUTNER = 128
B_CHECKER2:
        LCALL   CS_3
        MOV     A,#0FH       ; PUT 00001111 TO DATA BYTE
        LCALL   C_DW3
        DJNZ    R4,B_CHECKER2
        MOV     R4,#128      ; RE-SET X COUNTER = 128
        DJNZ    R3,B_CHECKER1
        MOV     COM,#0B2H    ; RESET AUTO
        LCALL   C_DW2
        RET

GRAPH:                       ; ( PROCEDRUE FOR DISPLAY IMAGE )
        MOV     A,O_Y
        MOV     B,#20H       ; CALCULATE THE RAM ADDRESS
        MUL     AB
        ADD     A,O_X
        MOV     O_X,A
        MOV     A,B
        ADDC    A,#08H
        MOV     O_Y,A
        MOV     DAT1,O_X
        MOV     DAT2,O_Y
        MOV     DPTR,#IMG
        MOV     COM,#24H     ; SET ADDRESS POINTER
        LCALL   C_DW


        MOV     R3,#64       ; SET Y COUNTER = 64
        MOV     R4,#30       ; SET X COUNTER = 30
        MOV     COM,#0B0H
        LCALL   C_DW2
GRAPH1:
        LCALL   CS_3
        CLR     A
        MOVC    A,@A + DPTR  ; GET THE IMAGE DATA
        LCALL   FLIP         ; FLIP THE IMAGE DATA BTYE
        LCALL   C_DW3
        INC     DPTR
        DJNZ    R4,GRAPH1
        MOV     R4,#30       ; RE-SET THE X COUNTER = 30

        LCALL   CS_3
        MOV     A,#FFH       ; PUT 1111111 DATA BYTE IN LAST TWO COLUMN
        LCALL   C_DW3
        LCALL   CS_3
        MOV     A,#FFH
        LCALL   C_DW3
        DJNZ    R3,GRAPH1

        MOV     COM,#0B2H    ; RESET AUTO
        LCALL   C_DW2
        RET

C_DW:                        ; 2 DATA AND 1 COMMAND WRITE
        LCALL   CS
        MOV     A,DAT1
        LCALL   C_DW3
C_DW1:                       ; 1 DATA AND 1 COMMAND WRITE
        LCALL   CS
        MOV     A,DAT2
        LCALL   C_DW3
C_DW2:                       ; 1 COMMNAD WRITE ONLY
        LCALL   CS
        MOV     A,COM
        LJMP    C_DW4
C_DW3:
        CLR     C/D          ; COMMAND OR DATA SELECTION
C_DW4:
        MOV     P1,A
        CLR     WR
        SETB    WR
        RET

CS:                          ; CHECK STATUS
        LCALL   R_STATUS
        JNB     ACC.0,CS     ; CHECK COMMAND EXECUTION CAPABILITY
        JNB     ACC.1,CS     ; CHECK DATA READ / WRITE CAPABILITY
        RET

CS_2:
        LCALL   R_STATUS
        JNB     ACC.2,CS_2   ; CHECK AUTO MODE DATA READ CAPABILITY
        RET

CS_3:
        LCALL   R_STATUS
        JNB     ACC.3,CS_3   ; CHECK AUTO MODE DATA WRITE CAPABILITY
        RET

R_STATUS:
        MOV     P1,#0FFH     ; READ THE STATUS
        SETB    C/D
        CLR     RD
        MOV     A,P1
        SETB    RD
        RET

BUTTON: JB      CLICK,$      ; ( PROCEDURE FOR DETECT CLICKING OF PUSH BUTTON )
        ACALL   WAIT         ; ESCAPE FROM THE DEBOUNCING OF BUTTON
        JNB     CLICK,$
        ACALL   WAIT
        RET

FLIP:
        PUSH    7
        MOV     R7,#8
FLIP1:
        RLC     A
        MOV     TEMP3,A
        MOV     A,TEMP4
        RRC     A
        MOV     TEMP4,A
        MOV     A,TEMP3
        DJNZ    R7,FLIP1
        MOV     A,TEMP4
        POP     7
        RET

DELAY:                       ; ( PROCEDURE FOR TIME DELAY )
        PUSH    5
        PUSH    6
        PUSH    7
        MOV     R5,#150
DL1:    MOV     R6,#20
DL2:    MOV     R7,#50
DL3:    DJNZ    R7,DL3
        DJNZ    R6,DL2
        DJNZ    R5,DL1
        POP     7
        POP     6
        POP     5
        RET

WAIT:                        ; ( PROCEDURE FOR TIME WAITING )
        PUSH    6
        PUSH    7
        MOV     R6,#100
WAIT2:  MOV     R7,#200
        DJNZ    R7,$
        DJNZ    R6,WAIT2
        POP     7
        POP     6
        RET

IMG:            DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,FEH,00H,00H,00H,00H,00H,00H;
        DB 00H,7CH,00H,00H,C0H,01H,00H,00H;
        DB 00H,00H,0EH,00H,80H,CFH,63H,60H;
        DB 00H,00H,00H,00H,00H,00H,00H,DFH;
        DB 01H,00H,00H,00H,00H,00H,00H,30H;
        DB 00H,00H,80H,01H,00H,00H,00H,00H;
        DB 0CH,00H,00H,83H,03H,60H,00H,00H;
        DB 00H,00H,00H,00H,80H,FFH,01H,00H;
        DB 00H,00H,00H,00H,00H,18H,00H,00H;
        DB C0H,00H,00H,00H,00H,00H,06H,00H;
        DB 80H,E1H,00H,30H,00H,00H,00H,00H;
        DB 00H,00H,80H,FFH,03H,00H,00H,00H;
        DB 00H,00H,00H,18H,00H,00H,C0H,00H;
        DB 00H,00H,00H,00H,06H,00H,80H,71H;
        DB 00H,30H,00H,00H,00H,00H,00H,00H;
        DB 00H,FFH,03H,00H,00H,00H,00H,00H;
        DB 00H,18H,C0H,07H,FEH,F0H,DEH,03H;
        DB 3EH,F0H,07H,00H,80H,1DH,3CH,FCH;
        DB 00H,00H,00H,00H,00H,00H,00H,FFH;
        DB 33H,00H,00H,00H,00H,00H,00H,0CH;
        DB 60H,0CH,73H,F8H,F3H,06H,73H,98H;
        DB 03H,00H,C0H,0EH,3EH,30H,00H,00H;
        DB 00H,00H,00H,00H,F0H,FFH,FFH,01H;
        DB 00H,00H,00H,00H,00H,0CH,60H,0EH;
        DB 73H,D8H,71H,06H,63H,98H,03H,00H;
        DB C0H,0FH,36H,18H,00H,00H,00H,00H;
        DB 00H,00H,CEH,7FH,03H,07H,00H,00H;
        DB 00H,00H,00H,0CH,30H,80H,71H,D8H;
        DB 39H,86H,61H,8CH,03H,00H,C0H,0DH;
        DB 30H,18H,00H,00H,00H,00H,00H,80H;
        DB BBH,7FH,1FH,0FH,00H,00H,00H,00H;
        DB 00H,06H,30H,80H,31H,E0H,18H,83H;
        DB 61H,8CH,01H,00H,60H,18H,18H,18H;
        DB 00H,00H,00H,00H,00H,C0H,DCH,E8H;
        DB 31H,1FH,00H,00H,00H,00H,00H,86H;
        DB 33H,86H,31H,E3H,18H,83H,71H,8CH;
        DB 19H,00H,60H,18H,D8H,6CH,00H,00H;
        DB 00H,00H,00H,60H,72H,24H,C2H,34H;
        DB 00H,00H,00H,00H,00H,86H,31H,86H;
        DB B9H,61H,18H,9BH,31H,CCH,0DH,00H;
        DB 60H,18H,D8H,ECH,00H,00H,00H,00H;
        DB 00H,B0H,69H,22H,E4H,68H,00H,00H;
        DB 00H,00H,80H,FFH,E1H,03H,FFH,61H;
        DB 0CH,0FH,1FH,F8H,0FH,00H,F8H,78H;
        DB 78H,3CH,00H,00H,00H,00H,00H,D8H;
        DB 84H,23H,BCH,D0H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,6CH,82H,3FH;
        DB 07H,A1H,01H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,36H,C1H,F8H,04H,22H;
        DB 03H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,12H,41H,08H,08H,42H,03H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,8BH;
        DB 20H,08H,08H,C4H,06H,00H,00H,40H;
        DB 00H,00H,00H,10H,02H,00H,00H,00H;
        DB 04H,80H,00H,00H,00H,09H,04H,00H;
        DB 00H,00H,00H,00H,00H,4DH,7CH,08H;
        DB 88H,8FH,04H,00H,00H,40H,20H,02H;
        DB 00H,10H,02H,00H,00H,00H,04H,80H;
        DB 00H,00H,00H,09H,20H,00H,00H,00H;
        DB 00H,00H,80H,44H,D6H,0CH,68H,88H;
        DB 0CH,00H,00H,40H,20H,02H,00H,08H;
        DB 01H,00H,00H,00H,04H,80H,00H,00H;
        DB 00H,09H,20H,00H,00H,00H,00H,00H;
        DB 80H,E2H,FDH,04H,38H,10H,09H,00H;
        DB 00H,C0H,F3H,EFH,21H,08H,49H,92H;
        DB 24H,09H,C4H,F1H,BCH,C3H,E1H,89H;
        DB F4H,C0H,71H,BCH,03H,00H,40H,62H;
        DB FEH,05H,F8H,13H,19H,00H,00H,40H;
        DB 24H,22H,22H,84H,48H,92H,24H,09H;
        DB 24H,8AH,44H,24H,12H,49H,24H,20H;
        DB 8AH,44H,04H,00H,40H,61H,FFH,05H;
        DB F8H,17H,11H,00H,00H,40H,24H,22H;
        DB 02H,84H,A8H,52H,A5H,0AH,24H,88H;
        DB 44H,24H,12H,29H,24H,20H,88H,44H;
        DB 04H,00H,20H,B1H,FFH,03H,FCH,17H;
        DB 12H,00H,00H,40H,24H,22H,02H,42H;
        DB A8H,52H,A5H,0AH,24H,88H,44H,24H;
        DB 12H,39H,24H,20H,88H,44H,04H,00H;
        DB A0H,B0H,FFH,03H,BCH,17H,22H,00H;
        DB 00H,40H,24H,22H,22H,42H,10H,21H;
        DB 42H,84H,24H,8AH,44H,24H,12H,49H;
        DB 24H,24H,8AH,44H,04H,00H,90H,B8H;
        DB 8FH,03H,1CH,17H,20H,00H,00H,40H;
        DB C4H,ECH,21H,21H,10H,21H,42H,84H;
        DB C4H,F1H,44H,C4H,E1H,89H,C4H,C4H;
        DB 71H,44H,04H,00H,50H,A8H,87H,03H;
        DB 1CH,3FH,64H,00H,00H,00H,00H,20H;
        DB 00H,21H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,58H,C8H,07H,03H,10H,2FH;
        DB 44H,00H,00H,00H,00H,20H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 28H,C8H,07H,32H,90H,23H,44H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,28H,04H;
        DB 07H,50H,00H,20H,48H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,24H,04H,01H,59H;
        DB 00H,20H,C8H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,24H,84H,00H,9DH,00H,00H;
        DB 88H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 14H,84H,00H,3FH,01H,40H,80H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,16H,84H;
        DB 00H,7FH,01H,40H,90H,01H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,12H,00H,04H,81H;
        DB 03H,40H,80H,01H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,12H,80H,84H,01H,07H,46H;
        DB 10H,01H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 12H,80H,0CH,FFH,07H,4AH,10H,01H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,12H,00H;
        DB 3CH,01H,10H,45H,00H,01H,30H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,C0H,00H,00H;
        DB 00H,00H,00H,00H,12H,82H,FCH,01H;
        DB 90H,45H,00H,01H,30H,00H,00H,3FH;
        DB 00H,00H,00H,30H,08H,00H,38H,00H;
        DB 3EH,06H,00H,C0H,00H,00H,00H,0CH;
        DB 03H,00H,12H,82H,FEH,1FH,9CH,42H;
        DB 10H,01H,30H,00H,00H,63H,00H,00H;
        DB 00H,70H,08H,00H,6CH,00H,43H,06H;
        DB 00H,C0H,00H,00H,00H,0CH,03H,00H;
        DB 02H,80H,C1H,3FH,D4H,02H,10H,01H;
        DB F0H,31H,03H,63H,3CH,3CH,1EH,D0H;
        DB 88H,0FH,6CH,00H,03H,9FH,C7H,C7H;
        DB 87H,C7H,07H,98H,31H,03H,22H,80H;
        DB 07H,3EH,7CH,01H,10H,01H,30H,33H;
        DB 03H,63H,60H,46H,33H,D0H,C8H,0CH;
        DB 6CH,00H,07H,C6H,CCH,CCH,CCH,CCH;
        DB 0CH,F0H,30H,03H,22H,00H,79H,F8H;
        DB 38H,21H,10H,01H,30H,33H,03H,3FH;
        DB 7CH,06H,33H,90H,C9H,0CH,38H,03H;
        DB 3EH,C6H,CCH,CCH,CCH,CCH,0CH,60H;
        DB 30H,03H,26H,00H,81H,C3H,BFH,20H;
        DB 80H,01H,30H,E3H,01H,1BH,66H,06H;
        DB 3FH,10H,CBH,0CH,6CH,01H,70H,C6H;
        DB CFH,CCH,CCH,CFH,0CH,60H,30H,03H;
        DB 24H,04H,81H,38H,5FH,00H,88H,01H;
        DB 30H,E3H,01H,33H,66H,06H,03H,10H;
        DB CBH,0CH,CCH,00H,60H,C6H,C0H,CCH;
        DB CCH,C0H,0CH,60H,30H,03H,44H,04H;
        DB 01H,C0H,29H,20H,88H,00H,30H,E3H;
        DB 01H,63H,66H,46H,23H,10H,CEH,0CH;
        DB 8CH,01H,61H,C6H,C8H,CCH,CCH,C8H;
        DB 0CH,60H,30H,03H,4CH,04H,01H,01H;
        DB 27H,20H,C4H,00H,F0H,C1H,00H,C3H;
        DB 7CH,3CH,1EH,10H,8CH,0FH,78H,03H;
        DB 3EH,9CH,C7H,C7H,8CH,C7H,0CH,60H;
        DB E0H,03H,08H,04H,01H,01H,18H,10H;
        DB C4H,00H,00H,C0H,00H,00H,00H,00H;
        DB 00H,00H,00H,0CH,00H,00H,00H,00H;
        DB C0H,00H,00H,00H,00H,00H,00H,00H;
        DB 98H,00H,02H,01H,08H,10H,40H,00H;
        DB 00H,60H,00H,00H,00H,00H,00H,00H;
        DB 80H,07H,00H,00H,00H,00H,C0H,00H;
        DB 00H,00H,00H,00H,00H,00H,98H,08H;
        DB 02H,01H,08H,10H,62H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,30H,09H,02H,01H;
        DB 08H,08H,23H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,60H,10H,00H,01H,08H,08H;
        DB 31H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB C0H,12H,04H,02H,00H,88H,1CH,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,80H,07H;
        DB 00H,02H,04H,44H,06H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,5EH,08H,02H;
        DB 04H,F4H,03H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,70H,10H,04H,02H,3EH;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,C0H,13H,04H,86H,03H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 7EH,F8H,FFH,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,E0H,FFH;
        DB 01H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,80H,27H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;
        DB 00H,00H,00H,00H,00H,00H,00H,00H;




        END
